@@ -100,13 +100,16 @@ class HelpDesk extends CRMEntity {
 
 	/**	Constructor which will set the column_fields in this object
 	 */
+        function __construct() {
+            $this->log =Logger::getLogger('helpdesk');
+            $this->log->debug("Entering HelpDesk() method ...");
+            $this->db = PearDatabase::getInstance();
+            $this->column_fields = getColumnFields('HelpDesk');
+            $this->log->debug("Exiting HelpDesk method ...");
+        }   
 	function HelpDesk()
 	{
-		$this->log =LoggerManager::getLogger('helpdesk');
-		$this->log->debug("Entering HelpDesk() method ...");
-		$this->db = PearDatabase::getInstance();
-		$this->column_fields = getColumnFields('HelpDesk');
-		$this->log->debug("Exiting HelpDesk method ...");
+            self::__construct();
 	}
 
 
