@@ -511,7 +511,7 @@ class Settings_Profiles_Record_Model extends Settings_Vtiger_Record_Model {
 				foreach ($availableActionIds as $actionId) {
 					if ($actionId === 0) {
 						//Save action permissions = Permissions of Create/Editview action
-						$actionPermissions[$actionId] = $actionPermissions[1];
+						$actionPermissions[$actionId] = $actionPermissions[1] || $actionPermissions[7];
 					} else {
 						$actionPermissions[$actionId] = $actionPermissions[$actionId];
 					}
@@ -571,11 +571,13 @@ class Settings_Profiles_Record_Model extends Settings_Vtiger_Record_Model {
 					//Standard permissions
 					$i = 0;
 					$count = count($actionsIdsList);
-					$actionsInsertQuery .= 'INSERT INTO vtiger_profile2standardpermissions(profileid, tabid, operation, permissions) VALUES ';
+                    $actionParams = array();
+					$actionsInsertQuery = 'INSERT INTO vtiger_profile2standardpermissions(profileid, tabid, operation, permissions) VALUES ';
 					foreach ($actionsIdsList as $actionId => $permission) {
 						$actionEnabled = true;
 						$permissionValue = $this->tranformInputPermissionValue($permission);
-						$actionsInsertQuery .= "($profileId, $tabId, $actionId, $permissionValue)";
+						$actionsInsertQuery .= '(?, ?, ?, ?)';
+                        array_push($actionParams, $profileId, $tabId, $actionId, $permissionValue);
 
 						if ($i !== $count-1) {
 							$actionsInsertQuery .= ', ';
@@ -583,16 +585,18 @@ class Settings_Profiles_Record_Model extends Settings_Vtiger_Record_Model {
 						$i++;
 					}
 					if ($actionsIdsList) {
-						$db->pquery($actionsInsertQuery, array());
+						$db->pquery($actionsInsertQuery, $actionParams);
 					}
 
 					//Utility permissions
 					$i = 0;
 					$count = count($utilityIdsList);
-					$utilityInsertQuery .= 'INSERT INTO vtiger_profile2utility(profileid, tabid, activityid, permission) VALUES ';
+                    $utilityParams = array();
+					$utilityInsertQuery = 'INSERT INTO vtiger_profile2utility(profileid, tabid, activityid, permission) VALUES ';
 					foreach($utilityIdsList as $actionId => $permission) {
 						$permissionValue = $this->tranformInputPermissionValue($permission);
-						$utilityInsertQuery .= "($profileId, $tabId, $actionId, $permissionValue)";
+						$utilityInsertQuery .= '(?, ?, ?, ?)';
+                        array_push($utilityParams, $profileId, $tabId, $actionId, $permissionValue);
 
 						if ($i !== $count-1) {
 							$utilityInsertQuery .= ', ';
@@ -600,12 +604,9 @@ class Settings_Profiles_Record_Model extends Settings_Vtiger_Record_Model {
 						$i++;
 					}
 					if ($utilityIdsList) {
-						$db->pquery($utilityInsertQuery, array());
+						$db->pquery($utilityInsertQuery, $utilityParams);
 					}
 				}
-			} elseif ($this->isRestrictedModule($moduleModel->getName())) {
-				//To check the module is restricted or not(Emails, Webmails)
-				$actionEnabled = true;
 			}
 		} else {
 			$actionEnabled = true;
@@ -794,15 +795,6 @@ class Settings_Profiles_Record_Model extends Settings_Vtiger_Record_Model {
 		}
     }
 
-	/**
-	 * Function to check whether module is restricted for to show actions and field access
-	 * @param <String> $moduleName
-	 * @return <boolean> true/false
-	 */
-	public function isRestrictedModule($moduleName) {
-		return in_array($moduleName, array('Emails'));
-	}
-
 	/**
 	 * Function recalculate the sharing rules
 	 */
