@@ -385,7 +385,7 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	 * Function that returns all the fields for the module
 	 * @return <Array of Vtiger_Field_Model> - list of field models
 	 */
-	public function getFields() {
+	public function getFields($blockInstance=false) {
 		if(empty($this->fields)){
 			$moduleBlockFields = Vtiger_Field_Model::getAllForModule($this);
 			$this->fields = array();
@@ -593,17 +593,18 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	 * @return <Array> returns related fields list.
 	 */
 	public function getRelatedListFields() {
-		$entityInstance = CRMEntity::getInstance($this->getName());
-		$list_fields_name = $entityInstance->list_fields_name;
-		$list_fields = $entityInstance->list_fields;
 		$relatedListFields = array();
-		foreach ($list_fields as $key => $fieldInfo) {
-			foreach ($fieldInfo as $columnName) {
-				if(array_key_exists($key, $list_fields_name)){
-					$relatedListFields[$columnName] = $list_fields_name[$key];
+		$entityInstance = CRMEntity::getInstance($this->getName());
+		if (isset($entityInstance->list_fields_name)) {
+			$list_fields_name = $entityInstance->list_fields_name;
+			$list_fields = $entityInstance->list_fields;
+			foreach ($list_fields as $key => $fieldInfo) {
+				foreach ($fieldInfo as $columnName) {
+					if(array_key_exists($key, $list_fields_name)){
+						$relatedListFields[$columnName] = $list_fields_name[$key];
+					}
 				}
 			}
-
 		}
 		return $relatedListFields;
 	}
@@ -707,7 +708,7 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	 * @param <array> $restrictedModulesList
 	 * @return <array> List of module models <Vtiger_Module_Model>
 	 */
-	public static function getAll($presence = array(), $restrictedModulesList = array()) {
+	public static function getAll($presence = array(), $restrictedModulesList = array(),$sequenced = false) {
 		$db = PearDatabase::getInstance();
 		self::preModuleInitialize2();
 		$cacheKey = 'modules';
@@ -888,8 +889,10 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	 * @param <String> $where
 	 * @return <String> export query
 	 */
-	public function getExportQuery($where) {
-		$focus = CRMEntity::getInstance($this->getName());
+	public function getExportQuery($focus, $where) {
+		if(!$focus) {
+			$focus = CRMEntity::getInstance($this->getName());
+        }
 		$query = $focus->create_export_query($where);
 		return $query;
 	}
@@ -1089,6 +1092,14 @@ class Vtiger_Module_Model extends Vtiger_Module {
 					AND (vtiger_activity.status is NULL OR vtiger_activity.status NOT IN ('Completed', 'Deferred', 'Cancelled'))
 					AND (vtiger_activity.eventstatus is NULL OR vtiger_activity.eventstatus NOT IN ('Held','Cancelled'))";
 
+		if(!$currentUser->isAdminUser()) {
+			$moduleFocus = CRMEntity::getInstance('Calendar');
+			$condition = $moduleFocus->buildWhereClauseConditionForCalendar();
+			if($condition) {
+				$query .= ' AND '.$condition;
+			}
+		}
+
 		$params = array($this->getName());
 
 		if ($recordId) {
@@ -1241,6 +1252,7 @@ class Vtiger_Module_Model extends Vtiger_Module {
 					$params[] = $id;
 				}
 				if($subordinateUsers) {
+                    array_push($subordinateUsers, $currentUserModel->getId());
 					$ownerSql =  ' smownerid IN ('. implode(',' , $subordinateUsers) .')';
 				} else {
 					$ownerSql =  ' smownerid = '.$currentUserModel->getId();
@@ -1263,18 +1275,18 @@ class Vtiger_Module_Model extends Vtiger_Module {
 		$basicLinks = array();
 		if($createPermission) {
 			if($moduleName === "Calendar"){
-				$basicLinks[] = array(
-					'linktype' => 'BASIC',
-					'linklabel' => 'LBL_ADD_TASK',
-					'linkurl' => $this->getCreateTaskRecordUrl(),
-					'linkicon' => 'fa-plus'
-				);
 				$basicLinks[] = array(
 					'linktype' => 'BASIC',
 					'linklabel' => 'LBL_ADD_EVENT',
 					'linkurl' => $this->getCreateEventRecordUrl(),
 					'linkicon' => 'fa-plus'
 				);
+                $basicLinks[] = array(
+					'linktype' => 'BASIC',
+					'linklabel' => 'LBL_ADD_TASK',
+					'linkurl' => $this->getCreateTaskRecordUrl(),
+					'linkicon' => 'fa-plus'
+				);
 			} else {
 				$basicLinks[] = array(
 					'linktype' => 'BASIC',
@@ -1443,7 +1455,9 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	 * @return <String> - query
 	 */
 	public function getSearchRecordsQuery($searchValue,$searchFields, $parentId=false, $parentModule=false) {
-		return "SELECT ".implode(',',$searchFields)." FROM vtiger_crmentity WHERE label LIKE '%$searchValue%' AND vtiger_crmentity.deleted = 0";
+        $db = PearDatabase::getInstance();
+        $query = $db->convert2Sql("SELECT ".implode(',',$searchFields)." FROM vtiger_crmentity WHERE label LIKE ? AND vtiger_crmentity.deleted = 0", array("%$searchValue%"));
+		return $query;
 	}
 
 	/**
@@ -1521,6 +1535,14 @@ class Vtiger_Module_Model extends Vtiger_Module {
 
 		if ($nonAdminQuery) {
 			$query = appendFromClauseToQuery($query, $nonAdminQuery);
+
+			if($functionName == 'get_activities' && trim($nonAdminQuery)) {
+				$moduleFocus = CRMEntity::getInstance('Calendar');
+				$condition = $moduleFocus->buildWhereClauseConditionForCalendar();
+				if($condition) {
+					$query .= ' AND '.$condition;
+				}
+			}
 		}
 
 		return $query;
@@ -1593,96 +1615,125 @@ class Vtiger_Module_Model extends Vtiger_Module {
 	public function getRelatedModuleRecordIds(Vtiger_Request $request, $recordIds = array(), $nonAdminCheck = false) {
 		$db = PearDatabase::getInstance();
 		$relationIds = $request->get('related_modules');
-		if(empty($relationIds))  return array();
-
+        if(empty($relationIds))  return array();
+        
 		$focus = CRMEntity::getInstance($this->getName());
 		$relatedModuleMapping = $focus->related_module_table_index;
-
-		$relationFieldMapping = array();
-		$queryParams = array($this->getId());
-		foreach($relationIds as $reltionId) {
-			array_push($queryParams,$reltionId);
-		}
-		$query = "SELECT relationfieldid,related_tabid
-					FROM vtiger_relatedlists
-					WHERE vtiger_relatedlists.tabid=? AND relation_id IN (".generateQuestionMarks($relationIds).")";
-
-
-		$relationRes = $db->pquery($query,$queryParams);
-
-		$num_rows = $db->num_rows($relationRes);
-		for($i=0 ;$i<$num_rows; $i++) {
-			$relatedTabId = $db->query_result($relationRes,$i,'related_tabid');
-			$relationfieldid = $db->query_result($relationRes,$i,'relationfieldid');
-			$relatedModuleModel = Vtiger_Module_Model::getInstance($relatedTabId);
-			$relationFieldMapping[] = array('relatedModuleName'=>$relatedModuleModel->getName(),'relationfieldid'=>$relationfieldid);
-		}
-
+        
+        $relationFieldMapping = array();
+        $queryParams = array($this->getId());
+        foreach($relationIds as $reltionId) {
+            array_push($queryParams,$reltionId);
+        }
+        $query = "SELECT relationfieldid,related_tabid
+                    FROM vtiger_relatedlists
+                    WHERE vtiger_relatedlists.tabid=? AND relation_id IN (".generateQuestionMarks($relationIds).")";
+        
+        
+        $relationRes = $db->pquery($query,$queryParams);
+        
+        $num_rows = $db->num_rows($relationRes);
+        for($i=0 ;$i<$num_rows; $i++) {
+            $relatedTabId = $db->query_result($relationRes,$i,'related_tabid');
+            $relationfieldid = $db->query_result($relationRes,$i,'relationfieldid');
+            $relatedModuleModel = Vtiger_Module_Model::getInstance($relatedTabId);
+            $relationFieldMapping[] = array('relatedModuleName'=>$relatedModuleModel->getName(),'relationfieldid'=>$relationfieldid);
+        }
+        
 		$relatedIds = array();
 		if(!empty($relationFieldMapping)) {
-			foreach ($relationFieldMapping as $mappingDetails){
+            foreach ($relationFieldMapping as $mappingDetails){
 			//for ($i=0; $i<count($relatedModules); $i++) {
 				$params = array();
 				$module = $mappingDetails['relatedModuleName'];
-				$relationFieldId = $mappingDetails['relationfieldid'];
-				$sql = "SELECT vtiger_crmentity.crmid FROM vtiger_crmentity";
-
-				if($nonAdminCheck) {
-					if(empty($relatedModuleFocus)) $relatedModuleFocus = CRMEntity::getInstance($module);
-					$user = Users_Record_Model::getCurrentUserModel();
-					$relationAccessQuery = $relatedModuleFocus->getNonAdminAccessControlQuery($module, $user);
-					$sql .= ' '.$relationAccessQuery;
-				}
-
-				if(empty($relationFieldId)){
-					$tablename = $relatedModuleMapping[$module]['table_name'];
-					$tabIndex = $relatedModuleMapping[$module]['table_index'];
-					$relIndex = $relatedModuleMapping[$module]['rel_index'];
-
-					//Fallback to vtiger_crmentityrel if both focus and relationfieldid is empty
-					if(empty($tablename)) {
-						$tablename = 'vtiger_crmentityrel';
-						$tabIndex = 'crmid';
+                $relationFieldId = $mappingDetails['relationfieldid'];
+                $sql = "SELECT vtiger_crmentity.crmid FROM vtiger_crmentity";
+                
+                
+                if(empty($relationFieldId)){
+                    $tablename = $relatedModuleMapping[$module]['table_name'];
+                    $tabIndex = $relatedModuleMapping[$module]['table_index'];
+                    $relIndex = $relatedModuleMapping[$module]['rel_index'];
+					//To show related records comments in documents, should get related document records from vtiger_senotesrel.
+					if(empty($tablename) && $this->getName() == 'Documents') {
+						$tablename = 'vtiger_senotesrel';
+						$tabIndex =	'crmid';
+						$relIndex = 'notesid';
+					//To show related Document comments in current module
+					} else if (empty($tablename) && $module == 'Documents') {
+						$tablename = 'vtiger_senotesrel';
+						$tabIndex =	'notesid';
 						$relIndex = 'crmid';
-					}
-					//END
-
-					if($tablename == 'vtiger_crmentityrel'){
-						$sql .= " INNER JOIN $tablename ON ($tablename.relcrmid = vtiger_crmentity.crmid OR $tablename.crmid = vtiger_crmentity.crmid)
-							WHERE ($tablename.crmid IN (".  generateQuestionMarks($recordIds).")) OR ($tablename.relcrmid IN (".  generateQuestionMarks($recordIds)."))";
-						foreach ($recordIds as $key => $recordId) {
+					} else if(empty($tablename)) {
+						//Fallback to vtiger_crmentityrel if both focus and relationfieldid is empty
+                        $tablename = 'vtiger_crmentityrel';
+                        $tabIndex = 'crmid';
+                        $relIndex = 'crmid';
+                    } 
+                    //END
+                    
+                    if($tablename == 'vtiger_crmentityrel'){    
+                        $sql .= ' LEFT JOIN vtiger_activity ON vtiger_activity.activityid = vtiger_crmentity.crmid ';
+                        $sql .= " INNER JOIN $tablename ON ($tablename.relcrmid = vtiger_crmentity.crmid OR $tablename.crmid = vtiger_crmentity.crmid)
+                                  WHERE ($tablename.crmid IN (".  generateQuestionMarks($recordIds).") AND ($tablename.relmodule = '".$module."')) 
+                                    OR ($tablename.relcrmid IN (".  generateQuestionMarks($recordIds).") AND ($tablename.module = '".$module."'))";
+							foreach ($recordIds as $key => $recordId) {
 							array_push($params, $recordId);
+							}
+                    } else if($module == "Contacts" && $this->getName() == "Potentials"){
+                        $tablename = 'vtiger_contpotentialrel';
+                        $tabIndex = 'contactid';
+                        $sql .= ' LEFT JOIN vtiger_activity ON vtiger_activity.activityid = vtiger_crmentity.crmid ';
+                        $sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid 
+                        WHERE $tablename.potentialid IN (".  generateQuestionMarks($recordIds).")";
+                    } else {
+                        if(in_array($tablename,array('vtiger_senotesrel'))){
+                             $sql .= ' LEFT JOIN vtiger_activity ON vtiger_activity.activityid = vtiger_crmentity.crmid ';
+                        }
+                        $sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid
+                            WHERE $tablename.$relIndex IN (".  generateQuestionMarks($recordIds).")";
+                    }
+                }else{
+                    $fieldModel = Vtiger_Field_Model::getInstance($relationFieldId);
+                    $relatedModuleFocus = CRMEntity::getInstance($module);
+                    $tablename = $fieldModel->get('table');
+                    $relIndex = $fieldModel->get('column');
+                    if($tablename == $relatedModuleFocus->table_name){
+						if($this->getName() == "Contacts" && $module == "Potentials"){
+							$tablename = 'vtiger_contpotentialrel';
+							$tabIndex = 'potentialid';
+							$sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid
+                            WHERE $tablename.contactid IN (".  generateQuestionMarks($recordIds).")";
+						}else{
+							$tabIndex = $relatedModuleFocus->table_index;
+							$sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid
+                            WHERE $tablename.$relIndex IN (" . generateQuestionMarks($recordIds) . ")";
 						}
-					} else {
-						$sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid
-							WHERE $tablename.$relIndex IN (".  generateQuestionMarks($recordIds).")";
-					}
-				}else{
-					$fieldModel = Vtiger_Field_Model::getInstance($relationFieldId);
-					$relatedModuleFocus = CRMEntity::getInstance($module);
-					$tablename = $fieldModel->get('table');
-					$relIndex = $fieldModel->get('column');
-					if($tablename == $relatedModuleFocus->table_name){
-						$tabIndex = $relatedModuleFocus->table_index;
-						$sql .= " INNER JOIN $tablename ON $tablename.$tabIndex = vtiger_crmentity.crmid
-							WHERE $tablename.$relIndex IN (".  generateQuestionMarks($recordIds).")";
-					}else{
-						$modulePrimaryTableName = $relatedModuleFocus->table_name;
-						$modulePrimaryTableIndex = $relatedModuleFocus->table_index;
-						$tabIndex = $relatedModuleFocus->tab_name_index[$tablename];
-						$sql .= " INNER JOIN $modulePrimaryTableName ON $modulePrimaryTableName.$modulePrimaryTableIndex = vtiger_crmentity.crmid
-							INNER JOIN $tablename ON $tablename.$tabIndex = $modulePrimaryTableName.$modulePrimaryTableIndex 
-							WHERE $tablename.$relIndex IN (".  generateQuestionMarks($recordIds).")";
-					}
-				}
-
-				$sql .=' AND vtiger_crmentity.deleted = 0';
+                        
+                    }else{
+                        $modulePrimaryTableName = $relatedModuleFocus->table_name;
+                        $modulePrimaryTableIndex = $relatedModuleFocus->table_index;
+                        $tabIndex = $relatedModuleFocus->tab_name_index[$tablename];
+                        $sql .= " INNER JOIN $modulePrimaryTableName ON $modulePrimaryTableName.$modulePrimaryTableIndex = vtiger_crmentity.crmid
+                            INNER JOIN $tablename ON $tablename.$tabIndex = $modulePrimaryTableName.$modulePrimaryTableIndex 
+                            WHERE $tablename.$relIndex IN (".  generateQuestionMarks($recordIds).")";
+                    }
+                }
+				if($nonAdminCheck) {
+					$sqlComponents = explode(" WHERE ",$sql);
+                    if(empty($relatedModuleFocus)) $relatedModuleFocus = CRMEntity::getInstance($module);
+                    $user = Users_Record_Model::getCurrentUserModel();
+                    $relationAccessQuery = $relatedModuleFocus->getNonAdminAccessControlQuery($module, $user);
+					$sql = $sqlComponents[0].$relationAccessQuery." WHERE ".$sqlComponents[1];
+                }
+				
+                $sql .=' AND vtiger_crmentity.deleted = 0';
 				foreach ($recordIds as $key => $recordId) {
 					array_push($params, $recordId);
 				}
 
 				$result1 = $db->pquery($sql, $params);
-				$num_rows = $db->num_rows($result1);
+				$num_rows = $db->num_rows($result1); //should give doc crmid.
 				for($j=0; $j<$num_rows; $j++){
 					$relatedIds[] = $db->query_result($result1, $j, 'crmid');
 				}
@@ -1696,21 +1747,18 @@ class Vtiger_Module_Model extends Vtiger_Module {
 
 
 	public function transferRecordsOwnership($transferOwnerId, $relatedModuleRecordIds){
-		$moduleName = $this->getName();
 		foreach($relatedModuleRecordIds as $recordId) {
-			if(Users_Privileges_Model::isPermitted($moduleName, 'Save', $recordId)) {
-				try {
-					$recordModel = Vtiger_Record_Model::getInstanceById($recordId);
-					$recordModel->set('assigned_user_id', $transferOwnerId);
-					$recordModel->set('mode', 'edit');
-					// Transferring ownership with related module as Inventory modules, removes line item details.
-					// So setting $_REQUEST['ajxaction'] to DETAILVIEW
-					$_REQUEST['ajxaction'] = 'DETAILVIEW';
-					$recordModel->save();
-				} catch (DuplicateException $e) {
-					return $e->getDuplicationMessage();
-				} catch (Exception $e) {
-				}
+			try {
+				$recordModel = Vtiger_Record_Model::getInstanceById($recordId);
+				$recordModel->set('assigned_user_id', $transferOwnerId);
+				$recordModel->set('mode', 'edit');
+				// Transferring ownership with related module as Inventory modules, removes line item details.
+				// So setting $_REQUEST['ajxaction'] to DETAILVIEW
+				$_REQUEST['ajxaction'] = 'DETAILVIEW';
+				$recordModel->save();
+			} catch (DuplicateException $e) {
+				return $e->getDuplicationMessage();
+			} catch (Exception $e) {
 			}
 		}
 	}
